local M = {
  balance = 0,
  rating = 0,
  isSkipRide = false
}

local debug = false

local ratingTotal = 0
local ratingCount = 0

local refuelPrice = {
  gasoline = 2.0,
  diesel = 2.5,
  electricEnergy = 0.5
}

local repairVehiclePrice = 500

local originRefuelCarMethod = nil

local savedFuelInfo = nil

local function debugString(str)
  if debug then
    print(str)
  end
end

local function notifyBalanceChage()
  guihooks.trigger("TaxiDriver_BalanceChanged", { balance = M.balance })
end

local function notifyRatingChange()
  guihooks.trigger("TaxiDriver_RatingChanged", { rating = M.rating })
end

local function getFuelTotalPrice(liters, energyType)
  return liters * refuelPrice[energyType]
end

function M.monkeyPatch()
  local m = require('freeroam/facilities')
  originRefuelCarMethod = m.refuelCar

  m.refuelCar = function()
    local data = { position = be:getPlayerVehicle(0):getPosition() }
    guihooks.trigger("TaxiDriver_OnGasStation", data)
  end
end

function M.restoreMonkeyPatch()
  if not originRefuelCarMethod then
    return
  end

  local m = require('freeroam/facilities')
  m.refuelCar = originRefuelCarMethod
end

function M.payForFuel(liters, energyType)
  local vehicle = be:getPlayerVehicle(0)
  if not vehicle then
    return
  end

  local priceToPay = getFuelTotalPrice(liters, energyType)
  if M.balance - priceToPay < 0 then
    return
  end

  debugString(string.format('paying for fuel: %f liters, %f price', liters, priceToPay))

  M.balance = M.balance - priceToPay
  vehicle:queueLuaCommand(string.format('extensions["refuel-helper"].refuel(%f)', liters))
  guihooks.trigger("TaxiDriver_RefuelFinished", { price = priceToPay })
  notifyBalanceChage()
end

function M.finishRide(rideRating, rideCash)
  debugString(string.format('finishing ride: %f rating, %f cash', rideRating, rideCash))

  ratingTotal = ratingTotal + rideRating
  ratingCount = ratingCount + 1
  M.balance = M.balance + rideCash
  M.rating = ratingTotal / ratingCount

  debugString(string.format('rating: %f, balance: %f', M.rating, M.balance))

  guihooks.trigger("TaxiDriver_RideFinished", { rating = M.rating, cash = rideCash })
  notifyBalanceChage()
end

local function saveFuelInfo(data)
  savedFuelInfo = data[1]
end

local function repairVehicleJob(job)
  local vehicle = be:getPlayerVehicle(0)
  local position = vehicle:getPosition()
  local rotation = quatFromDir(vehicle:getDirectionVector())
  local activeCam = core_camera.getActiveCamNameByVehId(be:getPlayerVehicleID(0))
  local activeCamData = core_camera.getCameraDataById(be:getPlayerVehicleID(0))

  core_vehicleBridge.requestValue(vehicle, saveFuelInfo, 'energyStorage')

  ui_fadeScreen.start(0.3)
  job.sleep(3)

  core_vehicles.cloneCurrent()
  job.sleep(1.5)
  core_vehicles.removeAllExceptCurrent()

  local newVehicle = be:getPlayerVehicle(0)
  local newVehicleId = newVehicle:getID()

  job.sleep(1.5)
  spawn.safeTeleport(newVehicle, position, rotation, nil, nil, nil, nil, false)
  job.sleep(5)

  core_camera.setVehicleCameraByNameWithId(newVehicleId, activeCam, nil, activeCamData)

  while not savedFuelInfo do
    job.sleep(0.3)
  end

  for _, storageInfo in ipairs(savedFuelInfo) do
    core_vehicleBridge.executeAction(newVehicle, 'setEnergyStorageEnergy', storageInfo.name, storageInfo.currentEnergy)
  end

  savedFuelInfo = nil
  
  ui_fadeScreen.stop(1)

  guihooks.trigger("TaxiDriver_RepairFinished", { price = repairVehiclePrice })
  notifyBalanceChage()
end

function M.repairVehicle()
  local vehicle = be:getPlayerVehicle(0)
  if not vehicle then
    return
  end

  if M.balance - repairVehiclePrice < 0 then
    return
  end

  M.balance = M.balance - repairVehiclePrice
  core_jobsystem.create(repairVehicleJob)
end

function M.skipRide()
  M.isSkipRide = true
end

function M.rideSkipped()
  M.isSkipRide = false
end

function M.getBalanceAndRating()
  notifyBalanceChage()
  notifyRatingChange()
end

return M