local json = require('libs/lunajson/lunajson')
local Queue = require('lua/ge/extensions/TaxiDriverMod/queue')
math.randomseed(os.time())

local M = {}
local currentLocation = nil

local function loadJsonFileContent(path)
  local file = io.open(path, "rb") -- r read mode and b binary mode
  local content = file:read("*a") -- *a reads the whole file
  file:close()

  return json.decode(content)
end

local filePoints = nil

function M.loadPoints(levelName)
  filePoints = loadJsonFileContent(string.format('lua/ge/extensions/TaxiDriverMod/%s_coords.json', levelName))
end

function M.getSpawnPoint()
  if not filePoints then
    return nil
  end

  return filePoints.spawn.point
end

function M.getSpawnRotation()
  if not filePoints then
    return nil
  end

  return filePoints.spawn.rotation
end

function M.isFilePointsLoaded()
  return filePoints ~= nil
end

function M.chooseRandomLocation()
  local selectedLocation = nil
  local currentVec = nil
  if currentLocation ~= nil then
    currentVec = vec3(currentLocation.point[1], currentLocation.point[2], currentLocation.point[3])
  end

  local queue = Queue:new(30)
  local attempts = 0
  local maxAttempts = #filePoints.points
  while (attempts < maxAttempts)
  do
    attempts = attempts + 1

    local randomIndex = math.random(1, #filePoints.points)
    if (queue:contains(randomIndex) and attempts < maxAttempts) then
      goto continue
    end

    queue:push(randomIndex)

    selectedLocation = filePoints.points[randomIndex]
    if currentLocation == nil then
      break
    end

    local selectedVec = vec3(selectedLocation.point[1], selectedLocation.point[2], selectedLocation.point[3])
    if (selectedVec:distance(currentVec) > 80 or attempts >= maxAttempts) then
      break
    end

    ::continue::
  end

  currentLocation = selectedLocation
end

function M.getCurrentLocationPoint()
  if currentLocation == nil then
    return nil
  end

  return currentLocation.point
end

function M.getCurrentLocationRotation()
  if currentLocation == nil then
    return nil
  end

  return currentLocation.rotation
end

function M.notifyIsPointsLoaded()
  guihooks.trigger("TaxiDriver_IsPointsLoaded", { isLoaded = M.isFilePointsLoaded() })
end

return M
