local M = {
  moduleLookups = {}
}

M.values = {
  averageAcceleration = 0,
  averageDeceleration = 0,
  averageSideAcceleration = 0
}

local moduleName = "g-force-watcher"

local isRecording = false

local totalAcceleration = 0
local countAcceleration = 0

local totalDeceleration = 0
local countDeceleration = 0

local totalSideAcceleration = 0
local countSideAcceleration = 0

local framesCount = 0

local function updateAcceleration(value)
  totalAcceleration = totalAcceleration + value
  countAcceleration = countAcceleration + 1
end

local function updateDeceleration(value)
  totalDeceleration = totalDeceleration + value
  countDeceleration = countDeceleration + 1
end

local function updateSideAcceleration(value)
  totalSideAcceleration = totalSideAcceleration + value
  countSideAcceleration = countSideAcceleration + 1
end

local function updateAverages()
  if countAcceleration > 0 then
    M.values.averageAcceleration = totalAcceleration / countAcceleration
  end
  
  if countDeceleration > 0 then
    M.values.averageDeceleration = totalDeceleration / countDeceleration
  end
  
  if countSideAcceleration > 0 then
    M.values.averageSideAcceleration = totalSideAcceleration / countSideAcceleration
  end
end

local function updateGFX()
  if not isRecording then
    return
  end

  framesCount = framesCount + 1
  if framesCount < 30 then
    return
  end

  framesCount = 0

  local gravity = obj:getGravity()

  if gravity >= 0 then
    gravity = math.max(0.1, gravity)
  else
    gravity = math.min(-0.1, gravity)
  end
  
  local y = sensors.gy2 / -gravity
  local x = sensors.gx2 / -gravity
  
  if y <= -0.01 then
    updateAcceleration(math.abs(y))
  end
  if y >= 0.01 then
    updateDeceleration(y)
  end
  if math.abs(x) > 0.01 then
    updateSideAcceleration(math.abs(x))
  end
  
  updateAverages()
end

local function startRecording()
  totalAcceleration = 0
  countAcceleration = 0

  totalDeceleration = 0
  countDeceleration = 0

  totalSideAcceleration = 0
  countSideAcceleration = 0
  
  isRecording = true
end

local function stopRecording()
  isRecording = false
end

local function getGForceWatcherAverages()
  return M.values
end

local function onExtensionLoaded()
  M.moduleLookups.getGForceWatcherAverages = getGForceWatcherAverages
end

local function requestRegistration(gi)
  gi.registerModule(moduleName, {}, M.moduleLookups)
end

M.updateGFX = updateGFX
M.startRecording = startRecording
M.stopRecording = stopRecording
M.onExtensionLoaded = onExtensionLoaded
M.requestRegistration = requestRegistration

return M