local M = {}

local debug = false

local function debugString(str)
  if debug then
    print(str)
  end
end

local function getStorages(isNativeStorage)
	local res = {}
	for name, storage in pairs(energyStorage.getStorages()) do
		if storage.energyType == "gasoline" or storage.energyType == "diesel" or storage.energyType == "electricEnergy" then
			if isNativeStorage then
				table.insert(res, storage)
			else
				table.insert(res,
					{ name = name, capacity = storage.capacity, remainingVolume = storage.remainingVolume,
						energyType = storage.energyType })
			end
		end
	end

	return res
end

function M.refuel(liters)
	local storages = getStorages(true);
	local remainingLiters = liters
	for _, storage in pairs(storages) do
		local remainingVolume = storage.capacity - storage.remainingVolume

		debugString(string.format('refueling: %f liters, %f remainingVolume', remainingLiters, remainingVolume))

		if remainingVolume > 0 then
			local volumeToRefuel = math.min(remainingVolume, remainingLiters)
			if (storage.energyType == "electricEnergy") then
				debugString(string.format('refueling electric: %f liters, %f remainingVolume, %f stored energy', remainingLiters, remainingVolume, storage.storedEnergy))
				storage:setStoredEnergy(storage.storedEnergy + (volumeToRefuel * 3600000)) -- kWh to J
				remainingVolume = remainingVolume * 1000
			elseif (storage.energyType == "gasoline" or storage.energyType == "diesel") then
				storage:setRemainingVolume(storage.remainingVolume + volumeToRefuel)
			end

			remainingLiters = remainingLiters - volumeToRefuel
		end
	end
end

function M.notifyFuelCapacity()
	guihooks.trigger("TaxiDriver_FuelInfo", getStorages(false))
end

return M
