function getDistanceBetweenCoords(startPos, stopPos) {
  return Math.sqrt(
    Math.pow(stopPos.x - startPos.x, 2) +
      Math.pow(stopPos.y - startPos.y, 2) +
      Math.pow(stopPos.z - startPos.z, 2)
  );
}

angular.module("beamng.apps").directive("taxiDriverRefuel", [
  function () {
    return {
      templateUrl: "/ui/modules/apps/TaxiDriverRefuel/app.html",
      replace: false,
      restrict: "E",
      scope: false,
      controller: function ($scope, $element, $attrs) {
        this.isRefueling = false;
        this.refuelPosition = null;

        $scope.refuelPrice = {
          gasoline: 2.0,
          diesel: 2.5,
          electricEnergy: 0.5,
        };
        $scope.balance = 0;
        $scope.rating = 0;
        $scope.maxFuel = 0;
        $scope.selectedLiters = 0;
        $scope.energyType = null;
        $scope.energyMeasureValue = "Liters";
        $scope.isEnabled = false;

        const gasStationSectionAngular = angular.element(
          document.getElementById("taxi-driver-gas-station-section")
        );

        this.refuel = () => {
          if (!this.isRefueling) return;

          bngApi.engineLua(
            `extensions["TaxiDriverMod_TaxiDriver"].payForFuel(${$scope.selectedLiters}, "${$scope.energyType}")`
          );
        };
        this.repair = () => {
          if (!this.isRefueling) return;

          bngApi.engineLua(
            'extensions["TaxiDriverMod_TaxiDriver"].repairVehicle()'
          );
        };
        this.stopRefueling = () => {
          this.refuelPosition = null;
          this.isRefueling = false;

          $scope.energyType = null;
          $scope.selectedLiters = 0;

          gasStationSectionAngular.css({ display: "none" });
        };
        this.skipRide = () => {
          bngApi.engineLua(`extensions["TaxiDriverMod_TaxiDriver"].skipRide()`);
        };

        $scope.$on("TaxiDriver_OnGasStation", (ev, data) => {
          this.refuelPosition = data.position;
          this.isRefueling = true;
          bngApi.activeObjectLua(
            'extensions["refuel-helper"].notifyFuelCapacity()'
          );
        });
        $scope.$on("TaxiDriver_BalanceChanged", (ev, data) => {
          $scope.balance = data.balance;
        });
        $scope.$on("TaxiDriver_RideFinished", (ev, data) => {
          $scope.rating = data.rating;
        });
        $scope.$on("TaxiDriver_RatingChanged", (ev, data) => {
          $scope.rating = data.rating;
        });
        $scope.$on("TaxiDriver_RefuelFinished", (ev, data) => {
          this.stopRefueling();
        });
        $scope.$on("TaxiDriver_FuelInfo", (ev, data) => {
          gasStationSectionAngular.css({ display: "flex" });

          const totalCapacity = data.reduce((sum, t) => sum + t.capacity, 0);
          const currentCapacity = data.reduce(
            (sum, t) => sum + t.remainingVolume,
            0
          );

          $scope.maxFuel = totalCapacity - currentCapacity;
          $scope.energyType = data[0].energyType;
          gasStationSectionAngular.css({ display: "flex" });
        });

        $scope.$on("NavigationMapUpdate", (_, data) => {
          if (!this.isRefueling || !data) return;
          if (data && data.controlID && data.objects) {
            const obj = data.objects[data.controlID];
            const newPos = {
              x: obj.pos[0],
              y: obj.pos[1],
              z: obj.pos[2],
            };
            const distance = getDistanceBetweenCoords(
              this.refuelPosition,
              newPos
            );
            if (distance >= 1) {
              this.stopRefueling();
              return;
            }
          }
        });

        $scope.$on("TaxiDriver_IsPointsLoaded", (_, data) => {
          $scope.isEnabled = data.isLoaded;
        });

        bngApi.engineLua(
          `extensions["TaxiDriverMod_randomCoords"].notifyIsPointsLoaded()`
        );
        this.isEnabledInterval = setInterval(() => {
          bngApi.engineLua(
            `extensions["TaxiDriverMod_randomCoords"].notifyIsPointsLoaded()`
          );
        }, 60000);

        bngApi.engineLua(
          'extensions["TaxiDriverMod_TaxiDriver"].getBalanceAndRating()'
        );

        $scope.$on("$destroy", function () {
          clearInterval(this.isEnabledInterval);
        });
      },
      controllerAs: "ctrl",
    };
  },
]);
